// robinsonizer engine by Bob Polis
// copyright (c) 1994-2019

#ifndef __engine__
#define __engine__

#include <string>
#include <vector>
#include <ostream>
#include <random>
#include "robinsonizer_mode.hpp"

class engine {
    public:
        engine(const std::string& start,
               std::vector<std::string>&& numerals,
               int maxiter,
               robinsonizer_mode mode,
               unsigned int verbose);

        void run();
        void write_result(std::ostream& os) const;
        void frequencies(std::ostream& os, const int fm[]) const;
        void numeral_frequencies(std::ostream& os) const;
        unsigned long long total_iterations() const {return _total_iterations; }
        bool found() const { return _found; }

    protected:
        std::string                         _start;
        std::vector<std::string>            _numerals;
        int                                 _maxiter {10};
        robinsonizer_mode                   _mode {robinsonizer_mode::lax_autogram};
        unsigned long long                  _total_iterations {0};
        std::vector<std::vector<int>>       _frequencies; // for memory mgmt only
        std::vector<int*>                   _freq;
        std::vector<int>                    _start_freq;
        std::vector<int>                    _old;
        std::vector<int>                    _new;
        std::vector<int>                    _used;
        unsigned int                        _verbosity_level {0};
        bool                                _found {false};
        std::default_random_engine          _random_engine {std::random_device {}()};
        std::uniform_int_distribution<int>  _dist;
};

std::ostream& operator<<(std::ostream& os, const engine& engine);

#endif // __engine__
